@@ -1,6 +1,7 @@
 import os
 import shutil
 import json
+from collections import ChainMap
 
 # from functools import partial
 from typing import List, Optional
@@ -156,7 +157,8 @@ def populate_pkg_dir(
     )
     kwargs = {k: v for k, v in kwargs.items() if v is not None}
     # configs = dict(name=name, **configs, **kwargs, **args_defaults)
-    configs = dict(name=name, **args_defaults, **configs, **kwargs)
+    # configs = dict(name=name, **args_defaults, **configs, **kwargs)
+    configs = dict(ChainMap(dict(name=name), kwargs, configs, args_defaults))
 
     kwargs['description-file'] = kwargs.pop('description_file', '')
 
@@ -235,7 +237,11 @@ def update_pack_and_setup_py(
         shutil.copy(src=wads_join(resource_name), dst=pjoin(resource_name))
 
 
-if __name__ == '__main__':
+def main():
     import argh  # TODO: replace by argparse, or require argh in wads?
 
     argh.dispatch_command(populate_pkg_dir)
+
+
+if __name__ == '__main__':
+    main()
